!DIR$ FIXEDFORMLINESIZE:132
      include 'formats.f'
      include 'blocks.f'
      include 'Fourier.f'
      include 'index.f'
      include 'OptF.f'
      program Inv
      use formats
      use blocks
      implicit none

      real*8       chi , fchi
      integer*4    info, len
      logical*4    flag
      
      complex*16, allocatable, dimension(:) :: z
      real*8    , allocatable, dimension(:) :: rbnd
      integer*4 , allocatable, dimension(:) :: bing, bsol

      write(6,f1) '========================================================================='
      call check_pars(info)
      if(info .eq. 0) then
         write(6,f1) 'Inv: Parameters in "Inv.par" checked for sanity'
      else
         write(6,f3) 'Inv: There were errors in "Inv.par" file, info=', info, ' ; exiting'
         stop
      end if

      allocate(phi(1:L))
      call make_phi(info)
      if      (info .eq. 0) then
         write(6,f1) 'Inv: DFT data have been read from file "phi.dat"'
      else if(info .eq. -1) then
         write(6,f1) 'Inv: The value N in "Inv.par" is different from N in "phi.dat"'
         write(6,f1) 'Inv: Change N in "Inv.par" or use a different "phi.dat"; exiting'
         stop
      else if(info .eq. -2) then
         write(6,f1) 'Inv: The value r in "Inv.par" is different from r in "phi.dat"'
         write(6,f1) 'Inv: Change r in "Inv.par" or use a different "phi.dat"; exiting'
         stop
      else if(info .eq. -3) then
         write(6,f1) 'Inv: DFT data have been read from file "phi.dat"'
         write(6,f1) 'Inv: The number of lines in "phi.dat" is LARGER than L'
         write(6,f1) 'Inv: Not all data in "phi.dat" will be used'
         write(6,f1, advance='no') 'Inv: Continue [T/F]? : '
         read (5,f6)  flag
         if(flag) then
            continue
         else
            stop
         end if
      else if(info .eq. -4) then
         write(6,f1) 'Inv: The number of lines in file "phi.dat" is SMALLER than L'
         write(6,f1) 'Inv: Either decrease L or use another data file; exiting'
         stop
      else
         write(6,f2) 'Inv: There were errors reading file "phi.dat"; exiting, info=', info
         stop
      end if
      write(6,f1) '========================================================================='

      len = max(log10(dfloat(N))+2.0, 10.0)
      write(fv,f3) '(a, i', len, ', a)'
      write(6, fv) 'Inv:  N  =', N  , ' [  Length of the binary vector   ]'
      write(6, fv) 'Inv:  r  =', N1 , ' [        The popcount, r         ]'
      write(6, fv) 'Inv:  N-r=', N0 , ' [          N0 = N-r              ]'
      write(6, fv) 'Inv:  L  =', L  , ' [    Bandlimit of DFT data       ]'
      write(fv,f3) '(a, es', len, '.1, a)'
      write(6, fv) 'Inv: eps =', eps, ' [      Required precision        ]'
      write(6, f1) '========================================================================='

      allocate(rbnd(0:Nm1))
      allocate(bing(0:Nm1))
      allocate(bsol(0:Nm1))

      allocate(z(0:M))
      z      = cz
      z(0)   = dfloat(N1)
      z(1:L) = phi(1:L)
      call DFT_inv(z,rbnd)
      deallocate(z)
      call roughen(rbnd,bing)

      fname = trim('rig.dat')
      call dump_r(fname, rbnd)
      fname = trim('big.dat')
      call dump_b(fname, bing)

      IF     (Method .eq. 'CombSolver') THEN

         call check_Comb_pars(info)
         if(info .ge. 100) then
            write(6,f2) 'Inv: There were errors in "Comb.par", info=', info
            write(6,f1) 'Inv: Either remove "Comb.par" from working directory or fix it; exiting'
            stop
         else if(info .ge. 0) then
            write(6,f2) 'Inv: File "Comb.par" was read, number of parameters changed=', info
         end if

         write(6,f2) 'Inv: Starting CombSolver... Max recursion depth: ', depth_max
         write(6,f1) '-------------------------------------------------------------------------'
         call CombSolver(bing,bsol,info)
         write(6,f2) 'CombSolver completed, info=', info

      ELSE IF (Method .eq. 'OptFSolver') THEN

         call check_OptF_pars(info)
         if(info .ge. 100) then
            write(6,f2) 'Inv: There were errors in "OptF.par", info=', info
            write(6,f1) 'Inv: Either remove "OptF.par" from working directory or fix it; exiting'
            stop
         else if(info .ge. 0) then
            write(6,f2) 'Inv: File "OptF.par" was read, number of parameters changed=', info
         end if

         write(6,f1) 'Inv: Starting OptFSolver (Non-convex optimisation of F[x])...'
         write(6,f1) '-------------------------------------------------------------------------'
         call OptFSolver(rbnd,bsol,info)
         write(6,f2) 'OptFSolver info=', info
         if     (info .eq. -1) then
            write(6,f1) 'Inv: OptiFSolver failed to verify local minimum'
            write(6,f1) 'Inv: See User Guide Sec.9.5 re how to fix this error' 
         else if(info .eq. -2) then
            continue
         end if

      ELSE

         write(6,f1) 'Inv: Unexpected error: Wrong Method; exiting'
         stop

      END IF

      if(info .ge. 0) then
         allocate(z(0:M))
         call DFT_dir_b(1,L,bsol,z)
         chi = fchi(1,L,z,phi)
         deallocate(z)
      end if

      write(6,f1) '========================================================================='

      fv = '(a, es9.2)'
      if     (info .eq. 0) then

         write(6,f1) 'Inv: Binary vector with REQUIRED Fourier-space distance to data WAS found'
         write(6,fv) 'Inv: The solution has Fourier-space distance to data chi=', sngl(chi)

      else if(info .eq. 1) then

         write(6,f1) 'Inv: Binary vector with REQUIRED Fourier-space distance to data WAS NOT found'
         write(6,fv) 'Inv: However, a solution was found with chi=', sngl(chi)

      else

         write(6,f1) 'Inv: There were errors; solution WAS NOT found'
         write(6,f1) 'Inv: No output is written to any file'

      end if

      if(info .ge. 0) then

         fname = trim('sol.dat')
         call dump_b(fname, bsol)
         write(6,f1) 'Inv: Solution has been written to file "sol.dat" as N lines "i, x(i)"'

         inquire(file='mod.dat', exist=flag)
         if(flag) then

            write(6,f1) 'Inv: Working directory contains a file "mod.dat"...'
            write(6,f1) 'Inv: Will try to read it and compare the solution to the vector in "mod.dat"'
            call check_sol(bsol,info)
            if(info .ge. 0) then
               write(6,f2) 'Inv: Real-space distance between solution and "mod.dat", dist=', info
            else
               write(6,f2) 'Inv: There were problems interpreting "mod.dat", info=', info
            end if

         else

            write(6,f1) 'Inv: Working directory does not contains a file "mod.dat"...'
            write(6,f1) 'Inv: There is nothing to compare the solution to'

         end if

      end if

      write(6,f1) '========================================================================='


      stop
      end
