!DIR$ FIXEDFORMLINESIZE:132
      pure subroutine make_alpha(r,p,alpha,flag,info)
      use blocks
      implicit none
      real*8,    intent(in)  :: r(0:Nm1), p(0:Nm1)
      real*8,    intent(out) :: alpha
      logical*4, intent(out) :: flag
      integer*4, intent(out) :: info

      real*8    A0, A1, A2, F1, F2, F3, z(1:3)
      real*8    z1, z2, z3
      integer*4 k
      logical*4 RootsError
      
      INTERFACE

         pure subroutine make_cfc(r,p,A0,A1,A2)
         use blocks
         real*8, intent(in)  :: r(0:Nm1), p(0:Nm1)
         real*8, intent(out) :: A0,A1,A2
         end subroutine make_cfc

         pure subroutine solve_cubic(A0,A1,A2,z,k)
         use blocks
         real*8, intent(in)     :: A0, A1, A2
         real*8, intent(out)    :: z(1:3)
         integer*4, intent(out) :: k
         end subroutine solve_cubic

         pure subroutine solve_quadratic(A1,A2,z,k)
         use blocks
         real*8, intent(in)     :: A1, A2
         real*8, intent(out)    :: z(1:3)
         integer*4, intent(out) :: k
         end subroutine solve_quadratic

         pure subroutine check_roots(A0,A1,A2,z,k,flag)
         real*8   , intent(in)  :: A0, A1, A2
         integer*4, intent(in)  :: k
         real*8   , intent(in)  :: z(1:3)
         logical*4, intent(out) :: flag
         end subroutine check_roots

         pure subroutine make_F(r,F)
         use blocks
         real*8, intent(in)  :: r(0:Nm1)
         real*8, intent(out) :: F
         end subroutine make_F

      END INTERFACE

      call make_cfc(r,p,A0,A1,A2)

      if(dabs(A0) .lt. tiny) then             ! STA IF-1

         if(dabs(A1) .lt. tiny) then          ! STA IF-2

            if(dabs(A2) .lt. tiny) then       ! STA IF-3
C....................................... Case A0=A1=A2=0
C............ z1=z2=z3=0 ;  F(alpha) has 1 min @ alpha=0

               alpha = rz
               flag  = .false.
               info  = 1
               return

C................................... End Case A0=A1=A2=0

            else                              ! ELS IF-3

C.................................. Case A0=A1=0, A2=/=0
C....................................... z1=-A2, z2=z3=0
C.......... F(alpha) has 1 min @ -A2 & 1 sadle point @ 0 

               alpha = -A2
               flag  = .true.
               info  = 2
               return

C...... End Case A0=A1=0, A2=/=0 .......................

            end if                            ! END IF-3

         else                                 ! ELS IF-2
C...................... Case A0=0, A1=/=0, A2 not tested
C.................. z1,z2 from A1 + A2*z + z^2 = 0; z3=0
         
            call solve_quadratic(A1,A2,z,k)

            if(k .eq. 0) then ! No real roots to quadraic; 1 min @ alpha=z3=0

               alpha = rz
               flag = .false.
               info = 3
               return  

            else if(k .eq. 2) then ! There are two real roots z1,z2 to the quadratic 
         
               call make_F(r+z(1)*p,F1)
               call make_F(r+z(2)*p,F2)
               call make_F(r,       F3)

               if(z(1)*z(2) .lt. rz) then ! sng(z1)=-sgn(z2) ; 1 max @ alpha=z3=0 & 2 mins @ alpha=z1,z2 

                  if(F1 .ge. F3) then
                     alpha = rz
                     flag  = .false.
                     info  = -1
                     return
                  end if
                  if(F2 .ge. F3) then
                     alpha = rz
                     flag  = .false.
                     info = -2
                     return
                  end if
                  if(F1 .lt. F2) then
                     alpha = z(1)
                     flag  = .true.
                     info  = 4
                     return
                  else
                     alpha = z(2)
                     flag  = .true.
                     info  = 5
                     return
                  end if

               else ! sng(z1)=sgn(z2) ; 1 min @ z3=0 &  1 min + 1 max at alpha=z1,z2 

                  if(dabs(z(1)) .lt. dabs(z(2))) then

                     if(F1 .lt. F3) then ! Impossible: z1 should be a max
                        alpha = rz
                        flag  = .false.
                        info  = -3
                        return
                     end if
                     if(F1 .lt. F2) then ! Impossible: z1 should be a max
                        alpha = rz
                        flag  = .false.
                        info  = -4
                        return
                     end if
                     flag = .false. ! Not moving over a max
                     info = 6
                     return

                  end if

               end if

            end if
C... End Case A0=0, A1=/=0, A2 not tested ..............

         end if                               ! END IF-2

      else                                    ! ELS IF-1
C.... Case A0=/=0, A1 and A2 not tested ................

         call solve_cubic(A0,A1,A2,z,k)
         if(CheckCubeRoots) then
            call check_roots(A0,A1,A2,z,k,RootsError)
            if(RootsError) then
               alpha = rz
               flag = .false.
               info = -5
               return
            end if
         end if
      
         if(k .eq. 1) then ! Cubic has one real root z1 ; one min @ alpha=z1

            alpha = z(1)
            flag  = .true.
            info  = 7
            return

         else ! Cubic has 3 real roots z1, z2, z3

            if(z(1) .le. z(2)) then
               z1 = z(1)
               z2 = z(2)
            else
               z1 = z(2)
               z2 = z(1)
            end if

            if     (z(3) .lt. z1) then
               z3 = z2
               z2 = z1
               z1 = z(3)
            else if(z(3) .lt. z2) then
               z3 = z2
               z2 = z(3)
            else
               z3 = z(3)
            end if

            if(z2 .ge. rz) then
               alpha = z1
            else 
               alpha = z3
            end if

         end if

C.End Case A0=/=0, A1 and A2 not tested ................

      end if                                  ! END IF-1


      info = -100
      return
      end
